# -*- coding: utf-8 -*-
import urllib.parse

from PIL import Image, ImageEnhance
from obs import ObsClient

import os
import string
import random
import traceback

LOCAL_MOUNT_PATH = '/tmp/'
RUNTIME_CODE_ROOT = os.getenv('RUNTIME_CODE_ROOT')


def new_obs_client(context):
    ak = context.getAccessKey()
    sk = context.getSecretKey()
    region_id = context.getUserData('obs_region')
    obs_server = 'https://obs.{}.myhuaweicloud.com'.format(region_id)
    return ObsClient(access_key_id=ak, secret_access_key=sk, server=obs_server)


def get_obs_obj_info(event):
    record = event['Records'][0]
    if 's3' in record:
        s3 = record['s3']
        return s3['bucket']['name'], s3['object']['key']
    else:
        obs_info = record['obs']
        return obs_info['bucket']['name'], obs_info['object']['key']


def set_opacity(im, opacity):
    if im.mode != "RGBA":
        im = im.convert('RGBA')
    else:
        im = im.copy()
    alpha = im.split()[3]
    alpha = ImageEnhance.Brightness(alpha).enhance(opacity)
    im.putalpha(alpha)
    return im


# generate a temporary directory for downloading things
# from OBS and compress them.
def gen_local_download_path():
    letters = string.ascii_letters
    download_dir = LOCAL_MOUNT_PATH + ''.join(
        random.choice(letters) for i in range(16)) + '/'
    os.makedirs(download_dir)
    return download_dir


class WatermarkHandler:

    def __init__(self, context):
        self.logger = context.getLogger()
        self.obs_client = new_obs_client(context)
        self.output_bucket = context.getUserData('obs_output_bucket')
        self.download_dir = gen_local_download_path()

    def run(self, event):
        src_bucket, src_obj_name = get_obs_obj_info(event)
        object_key = urllib.parse.unquote_plus(src_obj_name)
        # download file uploaded by user from obs
        local_file = os.path.join(self.download_dir, object_key)
        self.download_file(src_bucket, src_obj_name, local_file)
        out_file_name, out_file = self.watermark_image(local_file, src_obj_name)
        self.upload_file_to_obs(out_file_name, out_file)

    def download_file(self, bucket, obj_name, local_file):
        resp = self.obs_client.getObject(bucket, obj_name, local_file)
        if resp.status < 300:
            self.logger.info('succeed to download object: %s', obj_name)
        else:
            error_info = 'failed to download object %s from obs, ' \
                         'errorCode: %s, errorMessage: %s, requestID: %s' % (
                             obj_name, resp.errorCode, resp.errorMessage,
                             resp.requestId)
            raise Exception(error_info)

    def upload_file_to_obs(self, obj_name, file):
        resp = self.obs_client.putFile(self.output_bucket, obj_name, file)
        if resp.status < 300:
            self.logger.info('succeed to upload object: %s', obj_name)
        else:
            error_info = 'failed to upload object %s from obs, ' \
                         'errorCode: %s, errorMessage: %s, requestID: %s' % (
                             obj_name, resp.errorCode, resp.errorMessage,
                             resp.requestId)
            raise Exception(error_info)

    def watermark(self, im, mark, opacity=0.6):
        if opacity < 1:
            mark = set_opacity(mark, opacity)
        if im.mode != 'RGBA':
            im = im.convert('RGBA')
        if im.size[0] < mark.size[0] or im.size[1] < mark.size[1]:
            error_info = "The mark image size is larger size than " \
                         "original image file."
            raise Exception(error_info)

        x = int((im.size[0] - mark.size[0]) / 2)
        y = int((im.size[1] - mark.size[1]) / 2)

        layer = Image.new('RGBA', im.size, )
        layer.paste(mark, (x, y))
        return Image.composite(layer, im, layer)

    def watermark_image(self, local_file, file_name):
        im = Image.open(local_file)
        watermark_image_path = os.path.join(RUNTIME_CODE_ROOT, "watermark.png")
        mark = Image.open(watermark_image_path)
        out = self.watermark(im, mark)
        self.logger.info("**********finish watermark**********")
        name = file_name.split('.')
        out_file_name = name[0] + '-watermark.' + name[1]
        out_file_path = os.path.join(self.download_dir, out_file_name)

        if out:
            out = out.convert('RGB')
            out.save(out_file_path)
        else:
            error_info = "Sorry, Save watermarked file Failed."
            raise Exception(error_info)

        return out_file_name, out_file_path

    def clean_local_files(self, file_path):
        if os.path.isfile(file_path):
            self.delete_local_file(file_path)
        elif os.path.isdir(file_path):
            for filename in os.listdir(file_path):
                sub_file_path = os.path.join(file_path, filename)
                if os.path.isfile(sub_file_path):
                    self.delete_local_file(sub_file_path)
                elif os.path.isdir(sub_file_path):
                    self.clean_local_files(sub_file_path)
                    os.rmdir(sub_file_path)

    def delete_local_file(self, file_path):
        try:
            os.remove(file_path)
        except:
            self.logger.error("failed to delete local file %s, exp:%s",
                              file_path, traceback.format_exc())


def handler(event, context):
    watermark_handler = WatermarkHandler(context)
    try:
        watermark_handler.run(event)
    finally:
        watermark_handler.obs_client.close()
        watermark_handler.clean_local_files(watermark_handler.download_dir)

    return 'OK'
